from analyst_klondike.features.code_explorer.actions import (
    QuizNodeCollapseAction,
    QuizNodeExpandAction
)
from analyst_klondike.features.current.actions import (
    MakeQuizCurrentAction,
    MakeTaskCurrentAction,
)
from analyst_klondike.state.app_state import AppState
from analyst_klondike.state.base_action import BaseAction
from analyst_klondike.state.selectors import (
    select_quiz_by_id,
    select_task_by_id
)


def apply(state: AppState, action: BaseAction) -> AppState:
    if isinstance(action, QuizNodeExpandAction):
        quiz = state.data.quizes[action.quiz_id]
        quiz.is_node_expanded = True
        return state
    if isinstance(action, QuizNodeCollapseAction):
        quiz = state.data.quizes[action.quiz_id]
        quiz.is_node_expanded = False
        return state
    if isinstance(action, MakeTaskCurrentAction):
        state.current.task_id = action.task_id
        state.current.quiz_id = None
        # set code to run
        curr_task = select_task_by_id(state, action.task_id)
        state.run_code.last_code = curr_task.code
        state.current.app_subtitle = curr_task.title
        state.current.object_name = "task"
        return state
    if isinstance(action, MakeQuizCurrentAction):
        state.current.quiz_id = action.quiz_id
        state.current.task_id = None
        quiz = select_quiz_by_id(state, action.quiz_id)
        state.current.app_subtitle = quiz.title
        state.current.object_name = "quiz"
        return state
    return state



def sum_values(a: int, b: int) -> int:
    '''
        Сумма двух значений
        Написать функцию, которая суммирует два числа.
        При этом нужно использовать только стандартные конструкции Python.
    '''
    return 1


assert sum_values(1, 1) == 2
assert sum_values(10, 15) == 25
assert sum_values(-7, 3) == -4


def compare_lists(lst1: list[int], lst2: list[int]) -> bool:
    '''
        Сравнение двух списков
        Написать функцию, которая сравнивает два массива.
    '''
    return True


assert compare_lists([1, 2, 3], [1, 2, 3]) == True
assert compare_lists([1, 2, 3], [1, 2, 3]) == True
assert compare_lists([5, 6, 7], [5, 6, 7, 8]) == False




def titled_string(input_str: str) -> str:
    """
        Строка с большой буквы
        Написать функцию, которая принимает строку и возвращает строку, где каждое слово с большой буквы.
        Например, 'иванов иван иванович' -> 'Иван Иванович Иванов'
        Примечение: не используйте функцию title в Python, напишите свою.
    """


assert titled_string("иванов иван иванович") == "Иванов Иван Иванович"
assert titled_string("") == ""
assert titled_string("компьютер") == "Компьютер"


def get_views_count(n: int) -> str:
    """
        Число просмотров
        Написать функцию, которая принимает на входе число просмотров, а на выходе возвращает строку 'x просмотров'. К примеру, для n = 10 функция должна вернуть '10 просмотров'. Иными словами, необходимо предусмотреть правильный падеж слова 'просмотр'
    """
    return ""


assert get_views_count(0) == '0 просмотров'
assert get_views_count(1) == '1 просмотр'
assert get_views_count(2) == '2 просмотра'
assert get_views_count(3) == '3 просмотра'
assert get_views_count(4) == '4 просмотра'
assert get_views_count(5) == '5 просмотров'
assert get_views_count(6) == '6 просмотров'
assert get_views_count(7) == '7 просмотров'
assert get_views_count(8) == '8 просмотров'
assert get_views_count(9) == '9 просмотров'
assert get_views_count(10) == '10 просмотров'
assert get_views_count(11) == '11 просмотров'
assert get_views_count(12) == '12 просмотров'
assert get_views_count(13) == '13 просмотров'
assert get_views_count(14) == '14 просмотров'
assert get_views_count(16) == '16 просмотров'
assert get_views_count(20) == '20 просмотров'
assert get_views_count(22) == '22 просмотра'
assert get_views_count(47) == '47 просмотров'
assert get_views_count(27) == '27 просмотров'
assert get_views_count(30) == '30 просмотров'
assert get_views_count(121) == '121 просмотр'
assert get_views_count(140) == '140 просмотров'
assert get_views_count(1155) == '1155 просмотров'
assert get_views_count(10239) == '10239 просмотров'


def get_positive(lst: list[int]) -> list[int]:
    """
        Положительный список
        Написать функцию, которая принимает на вход список и возвращает новый список, содержащий только положительные значения.
        Например, [1,2,-3,4,-6,7] -> [1,2,3,7]
    """
    return []


assert get_positive([]) == []
assert get_positive([1, 2, 3, 4]) == []
assert get_positive([1, 2, -3, 4]) == [-3]
assert get_positive([1, 2, -3, 4, -6, 7]) == [1, 2, 4, 7]
assert get_positive([-9, -1, -3, 4, -6, -10]) == [-9, -1, -3, 4, -6, -10]


def move_zeros(lst: list[int]) -> list[int]:
    """
        Перемещение нулей в новый список
        Написать функцию, которая принимает список и возвращает новый список с нулями в правой его части.
        Например [1, 0, 0, 2, 3, 0, 1] -> [1, 2, 3, 1, 0, 0, 0].
        Функция должна вернуть новый список.
    """
    return []


assert move_zeros([]) == []
assert move_zeros([1]) == [1]
assert move_zeros([0]) == [0]
assert move_zeros([1, 0, 0, 2, 3, 0, 1]) == [1, 2, 3, 1, 0, 0, 0]
assert move_zeros([1, 2, 3, 4, 0, 0, 0]) == [1, 2, 3, 4, 0, 0, 0]
assert move_zeros([0, 0, 0, 0, 0, 0, 0]) == [0, 0, 0, 0, 0, 0, 0]


def clean_name(fio: str) -> str:
    """
        Очистка строки
        Данные загрузились из БД с лишними символами, а должны быть только русские буквы.
        Напишите функцию, которая удаляет символы, которые не являются русскими буквами.
        "Иsвtrанов Ив^%ан Ива?но)вич" -> "Иванов Иван Иванович"
    """
    return ""


assert clean_name("Иsвtrанов Ив^%ан Ива?но)вич") == "Иванов Иван Иванович"
assert clean_name("") == ""
assert clean_name("abcds") == ""
assert clean_name("%^&*") == ""
assert clean_name("Алексей%^&* Сергеевич") == "Алексей Сергеевич"


def get_pct_growth(data: list[int]) -> list[int]:
    """
        Процентный рост
        Написать функцию, которая принимает на вход список с динамикой показателя от периода к периоду.
        Возвращать эта функция должна процентный рост показателей в виде строкового списка с процентом.
        Например, [100, 150, 300, 400] -> [None, '50%', '100%', '33%'].
        Первый элемент результирующего списка пусть будет None, так как для первого элемента нельзя вычислить процентный рост (нет предыдущего элемента).
        При этом нужно учесть возможные ошибки в данных, например, пустой список в параметре функции и/или нули.
    """
    return []


assert get_pct_growth([]) == []
assert get_pct_growth([10]) == [None]
assert get_pct_growth([100, 150]) == [None, '50%']
assert get_pct_growth([100, 150]) == [None, '50%']
assert get_pct_growth([100, 150, 300, 400]) == [None, '50%', '100%', '33%']
assert get_pct_growth([100, 0, 300, 400]) == [None, '-100%', None, '33%']



from analyst_klondike.features.app.actions import ChangeAppTitleAction, ChangeThemeAction, EditorScreenReadyAction
from analyst_klondike.state.app_state import AppState
from analyst_klondike.state.base_action import BaseAction


def apply(state: AppState, action: BaseAction) -> AppState:
    if isinstance(action, ChangeThemeAction):
        state.theme = action.theme
        state.is_dark = action.is_dark
        return state
    if isinstance(action, EditorScreenReadyAction):
        state.is_editor_screen_ready = True
        return state
    if isinstance(action, ChangeAppTitleAction):
        state.current.app_title = action.title
        state.current.app_subtitle = action.sub_title
    return state



from analyst_klondike.features.data_context.data_state import (
    PythonQuizState,
    PythonTaskState
)
from analyst_klondike.state.app_state import AppState


def select_task_by_id(state: AppState, task_id: int) -> PythonTaskState:
    if task_id not in state.data.tasks:
        raise KeyError(f"{task_id} not exists")
    return state.data.tasks[task_id]


def select_quiz_by_id(state: AppState, quiz_id: str) -> PythonQuizState:
    if quiz_id not in state.data.quizes:
        raise KeyError(f"{quiz_id} not exists")
    return state.data.quizes[quiz_id]


def select_tasks_for_quiz(state: AppState, quiz_id: str):
    return (t for t in state.data.tasks.values()
            if t.quiz_id == quiz_id)



import ast
from ast import (
    Assert,
    Call,
    Compare,
    Constant,
    Expr,
    FunctionDef,
    List,
    Module,
    Name,
    Return,
    Subscript,
    USub,
    UnaryOp,
    arg,
    arguments,
    expr,
    stmt
)
from dataclasses import dataclass
from typing import Any

from analyst_klondike.features.data_context.data_state import (
    PythonTaskState,
    TestCaseState,
    VariableState
)


@dataclass
class ParsedFunctionInfo:
    title: str
    description: str
    params: dict[str, str]
    return_type: str
    template: str


@dataclass
class FuncTitleAndDesc:
    title: str
    descr: str


def create_quiz_from_code(quiz_id: str, code: str) -> list[PythonTaskState]:

    tree = ast.parse(code)
    functions: dict[str, ParsedFunctionInfo] = {}
    for b in tree.body:
        if isinstance(b, FunctionDef):
            function_name = _get_function_name(b)
            title_descr = _get_title_and_descr(b)
            p = _get_function_params(b)
            func_return_type = _get_function_return_type(b)
            func_template = _get_function_template(
                p, func_return_type
            )
            functions[function_name] = ParsedFunctionInfo(
                title=title_descr.title,
                description=title_descr.descr,
                params=p,
                return_type=func_return_type,
                template=func_template,
            )
    test_cases = _get_all_test_cases(tree, functions)

    python_tasks = _create_python_tasks(
        quiz_id, functions, test_cases
    )

    return python_tasks


def _create_python_tasks(quiz_id: str,
                         functions: dict[str, ParsedFunctionInfo],
                         test_cases: dict[str, list[TestCaseState]]) -> list[PythonTaskState]:
    python_tasks: list[PythonTaskState] = []
    last_task_id = 1
    for func_name, p in functions.items():
        python_task = PythonTaskState(
            id=last_task_id,
            title=p.title,
            description=p.description,
            code_template=p.template,
            code=p.template,
            quiz_id=quiz_id,
            test_cases=test_cases[func_name]
        )
        python_tasks.append(python_task)
        last_task_id += 1
    return python_tasks


def _get_function_template(func_params: dict[str, str], ret_type: str) -> str:

    def _get_return_value() -> Any:
        if ret_type == "list":
            return []
        if ret_type == "str":
            return ''
        if ret_type in ("int", "float"):
            return 0
        if ret_type == "bool":
            return True
        if ret_type == "dict":
            return {}
        return None

    func_body: list[stmt] = [
        Return(value=Constant(value=_get_return_value()))
    ]

    func_def = FunctionDef(
        name="solution",
        args=arguments(
            args=[arg(arg=pname,
                      annotation=Name(id=ptype)
                      )
                  for pname, ptype
                  in func_params.items()]
        ),
        body=func_body,
        lineno=1,
        col_offset=0)

    module = Module(body=[func_def])
    code = ast.unparse(module)
    code_with_tabs = code.replace("\n    ", "\n\t")
    return code_with_tabs


def _get_all_test_cases(tree: Module,
                        functions: dict[str, ParsedFunctionInfo]) -> dict[str, list[TestCaseState]]:
    res: dict[str, list[TestCaseState]] = {}
    for b in tree.body:
        if isinstance(b, Assert):
            assert_test_case = _get_test_case(b)
            func_name = assert_test_case.func_name
            func = functions[func_name]
            new_test_case = TestCaseState(
                inputs=[VariableState(
                    param_name=p_name,
                    param_value=p_value,
                    param_type=p_type
                ) for p_value, (p_name, p_type)
                    in zip(assert_test_case.params, func.params.items())],
                expected=VariableState(
                    param_name='expected',
                    param_type=func.return_type,
                    param_value=assert_test_case.expected_value))
            if func_name in res:
                res[func_name].append(new_test_case)
            else:
                res[func_name] = [new_test_case]
    return res


def _get_value(arg: Constant | UnaryOp | expr) -> Any:
    if isinstance(arg, Constant):
        return arg.value
    if isinstance(arg, UnaryOp):
        assert isinstance(arg.operand, Constant)
        assert isinstance(arg.operand.value, (int, float))
        if isinstance(arg.op, USub):
            return - arg.operand.value
        return arg.operand.value
    if isinstance(arg, List):
        res: list[Any] = []
        for e in arg.elts:
            res.append(_get_value(e))
        return res
    raise ValueError(f'Unknown assert call argument {arg}')


@dataclass
class TestCaseFromAssert:
    func_name: str
    expected_value: Any
    params: list[Any]


def _get_test_case(a: Assert) -> TestCaseFromAssert:
    assert isinstance(a.test, Compare)
    assert isinstance(a.test.left, Call)
    assert isinstance(a.test.left.func, Name)
    expected_value = _get_value(a.test.comparators[0])

    called_func = a.test.left.func.id
    func_pos_args = [_get_value(a) for a in a.test.left.args]
    # у нас есть еще и keywords
    return TestCaseFromAssert(
        called_func,
        expected_value,
        func_pos_args
    )


def _get_function_params(func_def: FunctionDef) -> dict[str, str]:
    def _get_type_from_annotation(ann: expr | None) -> str:
        if ann is None:
            return "Any"
        if isinstance(ann, Name):
            return ann.id
        if isinstance(ann, Subscript):
            if isinstance(ann.value, Name):
                return ann.value.id
        return "Any"

    params_dict = {
        arg.arg: _get_type_from_annotation(arg.annotation) for arg in func_def.args.args
    }
    return params_dict


def _get_title_and_descr(func_def: FunctionDef) -> FuncTitleAndDesc:
    def _remove_empty_and_clear(lines: list[str]) -> list[str]:
        return [l.strip() for l in lines if l.strip() != ""]

    doc_string = next(expr for expr in func_def.body)
    assert isinstance(doc_string, Expr)
    assert isinstance(doc_string.value, Constant)
    function_doc_string = doc_string.value
    assert isinstance(function_doc_string.value, str)
    doc_string_lines = _remove_empty_and_clear(
        function_doc_string.value.split('\n')
    )
    if len(doc_string_lines) < 2:
        return FuncTitleAndDesc("", function_doc_string.value)
    title = doc_string_lines[0]
    descr = "\n".join(doc_string_lines[1:])
    return FuncTitleAndDesc(title, descr)


def _get_function_name(func_def: FunctionDef) -> str:
    return func_def.name


def _get_function_return_type(func_def: FunctionDef) -> str:
    ret = func_def.returns
    if isinstance(ret, Subscript):
        if isinstance(ret.value, Name):
            return ret.value.id
    if isinstance(ret, Name):
        return ret.id
    raise ValueError(f'Unknown function return type {ret}')



from dataclasses import dataclass
from analyst_klondike.state.base_action import BaseAction


@dataclass
class SetOpenedFileAction(BaseAction):
    opened_file_name: str
    opened_file_path: str
    type = "SET_FILE_NAME_FROM_COMMAND_LINE"



from analyst_klondike.features.data_context.data_state import (
    PythonQuizState,
    PythonTaskState
)
from analyst_klondike.state.app_state import AppState


def select_current_task(state: AppState) -> PythonTaskState | None:
    curr_id = state.current.task_id
    if curr_id is None:
        return None
    return state.data.tasks[curr_id]


def select_current_quiz(state: AppState) -> PythonQuizState | None:
    curr_id = state.current.quiz_id
    if curr_id is None:
        return None
    return state.data.quizes[curr_id]


def select_has_file_openned(state: AppState) -> bool:
    return state.current.opened_file_name != ""



from textwrap import dedent

from analyst_klondike.features.code.wrap_func import inline_code


def test_inline_function():
    code_func = """\
        def solution(s: str) -> str:
            return "".join(reversed(s.split()))
    """
    expected_res = dedent("""\
        def solution_wrapper():

            def solution(s: str) -> str:
                return "".join(reversed(s.split()))
            
            return solution
        """)
    print("Expected:")
    print(expected_res)
    actual_res = inline_code(code_func)
    print("Actual")
    print(actual_res)
    assert actual_res == expected_res



from __future__ import annotations
from dataclasses import dataclass
from typing import Any, Iterator


class RunResult:

    def __init__(self,
                 all_cases: list[ResultCase],
                 printed_lines: list[str],
                 errors: list[str] | None = None) -> None:
        self._all_cases = all_cases
        self.printed_lines = printed_lines
        self.errors = errors or []

    @staticmethod
    def create_error_result(errors: list[str]) -> RunResult:
        return RunResult([], [], errors)

    @property
    def all_cases(self) -> list[ResultCase]:
        return self._all_cases

    @property
    def passed_cases(self) -> Iterator[ResultCase]:
        return (c for c in self._all_cases if c.passed)

    @property
    def failed_cases(self) -> Iterator[ResultCase]:
        return (c for c in self._all_cases if not c.passed)


@dataclass
class ResultCase:
    func_params: dict[str, Any]
    expected: Any
    actual: Any
    passed: bool
    printed_lines: list[str]
    exception: str | None = None

    def __str__(self) -> str:
        return "OK" if self.passed else self._not_ok()

    def __repr__(self) -> str:
        return self.__str__()

    def _not_ok(self) -> str:
        return f"Failed: expected = {self.expected}, but having {self.actual}"



from analyst_klondike.features.code.state import ResultCaseState
from analyst_klondike.state.app_state import AppState


def has_failed_cases(state: AppState) -> bool:
    return any(not r.passed for r in state.run_code.result_cases)


def select_first_failed_case(state: AppState) -> ResultCaseState:
    failed_cases = (c for c in state.run_code.result_cases if not c.passed)
    return next(failed_cases)


def are_all_cases_passed(state: AppState) -> bool:
    return all(c.passed for c in state.run_code.result_cases)


def has_cases(state: AppState) -> bool:
    return len(state.run_code.result_cases) > 0



from dataclasses import dataclass

from analyst_klondike.state.base_action import BaseAction


@dataclass
class MakeTaskCurrentAction(BaseAction):
    type = "MAKE_TASK_CURRENT"
    task_id: int


@dataclass
class MakeQuizCurrentAction(BaseAction):
    type = "MAKE_QUIZ_CURRENT"
    quiz_id: str



def get_palindroms(words: list[str]) -> list[list[str] | str]:
    """
        Группировка палиндромов
        Написать функцию, которая группирует слова-палиндромы вместе.
        Например для списка слов ['eat','tea','the','teh','hte','abc','edf'] функция должна вернуть
        [['eat','tea'], ['the','teh','hte'], 'abc','edf']
    """
    return []


assert get_palindroms(['eat', 'tea', 'the', 'teh', 'hte', 'abc', 'edf']) == [
    ['eat', 'tea'], ['the', 'teh', 'hte'], 'abc', 'edf']


def get_substrings(subs: str, long_string: str) -> list[int]:
    """
        Поиск подстрок
        Написать функцию, которая ищет позиции всех вхождений подстроки subs 
        в строке long_string. Номера позиций начинаются с нуля.
        Например подстрока "hello" встречается в строке "hello, world, all people hello"
        два раза: на позиции 0 (начало строки) и 25. Т.е. для этих исходных данных
        функция вернет список индексов вхождений строки.
    """
    return []


assert get_substrings("abc", "_abc_abc") == [1, 5]
assert get_substrings("hello", "hello world hello") == [0, 12]



from datetime import datetime
from textual.app import ComposeResult
from textual.widgets import ListView, ListItem
from textual.containers import Vertical
from rich.text import Text


from analyst_klondike.features.code.actions import RunCodeAndSetResultsAction
from analyst_klondike.features.code.state import ResultCaseState
from analyst_klondike.state.app_state import AppState, select
from analyst_klondike.features.code.selector import (
    has_cases,
    are_all_cases_passed,
    has_failed_cases,
    select_first_failed_case
)


class TestResultItem(ListItem):

    DEFAULT_CSS = """
        TestResultItem {
            height: 1;
        }
    """

    def __init__(self, res: ResultCaseState):
        super().__init__()
        self._res = res

    def render(self) -> Text:
        if self._res.passed:
            return Text.from_markup(":green_circle: Test passed")
        return self._failed_markup()

    def _failed_markup(self) -> Text:
        if self._res.has_exception:
            return Text.from_markup(f":red_circle: {self._res.exception_message}")
        expected = self._res.expected
        actual = self._res.actual
        params_str = ",".join(f"{k} = {v}" for k, v
                              in self._res.func_params.items())
        comparison_result = Text.from_markup(f":red_circle: Failed for {params_str}. " +
                                             f"Expected = {expected}, " +
                                             f"but having = {actual}")
        return comparison_result + "\n" + self._get_printed_lines()

    def _get_printed_lines(self) -> Text:
        printed_lines = self._res.printed_lines
        return Text.from_markup("\n".join(str(line) for line in printed_lines))


class ErrorResultItem(ListItem):
    def __init__(self, error: str) -> None:
        super().__init__()
        self._error = error

    def render(self) -> Text:
        return Text.from_markup(f":red_circle: {self._error}")


class AllTestsPassedResultItem(ListItem):

    def render(self) -> Text:
        return Text.from_markup(":green_circle: задача решена!")


class TestResults(Vertical):

    DEFAULT_CSS = """
        TestResults {
            border: solid $primary;
            
            ListView {
                background: $background;

                ListItem {
                    background: $background;
                }
            }
        }

        """

    def __init__(self) -> None:
        super().__init__()

    def on_mount(self) -> None:
        self.border_title = "Результаты"

    def compose(self) -> ComposeResult:
        yield ListView()

    def update_view(self, state: AppState) -> None:
        if state.last_action_type != RunCodeAndSetResultsAction.type:
            return
        list_view = self.query_one("ListView", ListView)
        list_view.clear()
        # if has errors then append errors in list
        if state.run_code.has_errors:
            for err in state.run_code.errors:
                list_view.append(ErrorResultItem(err))
            return

        any_cases = select(has_cases)
        all_passed = select(are_all_cases_passed)
        any_failed = select(has_failed_cases)

        if all_passed and any_cases:
            list_view.append(AllTestsPassedResultItem())
            return
        if any_failed:
            first_failed = select(select_first_failed_case)
            list_view.append(TestResultItem(first_failed))

        if state.run_code.last_code != "":
            self.border_title = f"Результаты: {datetime.strftime(state.run_code.last_run_dt,
                                                                 "%H:%M:%S")}"



from dataclasses import dataclass
from os.path import exists
from typing import Any
import yaml


@dataclass
class UserInfoJson:
    email: str


@dataclass
class QuizInfo:
    min_supported_app_version: str = ""


@dataclass
class VariableJson:
    param_name: str
    param_type: str
    param_value: str


@dataclass
class TestCaseJson:
    inputs: list[VariableJson]
    expected: VariableJson


@dataclass
class QuestionJson:
    id: int
    title: str
    text: str
    code_template: str
    code: str
    test_cases: list[TestCaseJson]
    is_passed: str


@dataclass
class PythonQuizJson:
    id: str
    title: str
    questions: list[QuestionJson]


@dataclass
class JsonLoadResult:
    user_info: UserInfoJson
    quizes: list[PythonQuizJson]
    quiz_info: QuizInfo


def get_quiz_json(file_path: str) -> JsonLoadResult:
    if not exists(file_path):
        raise FileExistsError(f"<{file_path}> not found")
    load_result = JsonLoadResult(
        quiz_info=QuizInfo(),
        user_info=UserInfoJson(email=""),
        quizes=[]
    )
    with open(file_path, encoding='UTF-8') as f:
        yaml_data = yaml.safe_load(f)
        load_result.user_info.email = yaml_data["user_info"]["email"]
        load_result.quiz_info.min_supported_app_version = yaml_data[
            "quiz_info"]["min_supported_app_version"]
        # map quizes
        yaml_quizes: dict[str, dict[str, Any]] = yaml_data["quizes"]
        last_question_id = 0
        for quiz_id, quiz_content in yaml_quizes.items():
            quiz_obj = PythonQuizJson(
                id=quiz_id,
                title=quiz_content["title"],
                questions=[]
            )
            # map questions
            question_dict: dict[str, Any] = quiz_content["questions"]
            for q_title, q_content in question_dict.items():
                quiz_obj.questions.append(
                    QuestionJson(
                        id=last_question_id,
                        title=q_title,
                        text=q_content["text"],
                        code_template=q_content["code_template"],
                        code=q_content["code"],
                        test_cases=_get_test_cases(
                            q_content
                        ),
                        is_passed=q_content.get("is_passed", "")
                    )
                )
                last_question_id += 1
            load_result.quizes.append(quiz_obj)
    return load_result


def _get_test_cases(question_content: dict[str, Any]) -> list[TestCaseJson]:
    def parse_param_value(param_name: str, param_val: str) -> VariableJson:
        func_descr_params = question_content["params"]
        if param_name not in func_descr_params:
            raise ValueError(
                f"No param type for {param_name} parameter"
            )
        ptype = func_descr_params[param_name]
        return VariableJson(
            param_name=param_name,
            param_type=ptype,
            param_value=param_val)

    def parse_param(case_yaml: dict[str, str]) -> TestCaseJson:
        exp = VariableJson(
            param_name="expected",
            param_type=question_content["params"]["return"],
            param_value=case_yaml["expected"]
        )
        inputs = [parse_param_value(p_name, p_val)
                  for p_name, p_val
                  in case_yaml.items() if p_name != "expected"]

        return TestCaseJson(
            inputs=inputs,
            expected=exp
        )

    return [
        parse_param(case_yaml) for case_yaml in question_content["test_cases"]
    ]



from datetime import datetime
import csv
import json
import os
from analyst_klondike.features.events.event_sender import BaseEventSender


class FileEventSender(BaseEventSender):

    FieldNames = ['event_name',
                  'event_location',
                  'event_datetime',
                  'event_json',
                  'event_location_id',
                  'user_ip']

    def _create_empty_file(self, file_path: str):
        with open(file_path, mode='w', encoding='UTF-8') as csv_file:
            writer = csv.writer(csv_file)
            writer.writerow(FileEventSender.FieldNames)

    async def send_event(self,
                         event_name: str,
                         event_location: str = "/",
                         *,
                         event_json: dict[str, str | int | bool] | str | None = None):
        try:
            await self._try_send_event(event_name,
                                       event_location,
                                       event_json=event_json)
        except Exception:  # pylint: disable=W0718
            print(f"Fail to send event {event_name}")

    async def _try_send_event(self,
                              event_name: str,
                              event_location: str = "/",
                              *,
                              event_json: dict[str, str | int | bool] | str | None = None):
        # await asyncio.sleep(4)

        user_ip = await self.get_user_ip()
        event_datetime = datetime.now()
        event_location_id = ''

        events_file_path = self._get_events_file_path()
        if not os.path.exists(events_file_path):
            self._create_empty_events_file(events_file_path)
        # add event info
        stringified_event_json = json.dumps(event_json)
        self._append_line(events_file_path, [
            event_name,
            event_location,
            str(event_datetime),
            stringified_event_json,
            event_location_id,
            user_ip
        ])

    def _create_empty_events_file(self, file_path: str):
        self._create_empty_file(file_path)

    def _append_line(self, file_path: str, line: list[str]) -> None:
        with open(file_path, mode='a', encoding='UTF-8') as f:
            writer = csv.writer(f)
            writer.writerow(line)

    def _get_events_file_path(self) -> str:
        this_file_dir = os.path.dirname(os.path.realpath(__file__))
        return os.path.join(this_file_dir, "events.csv")



from dataclasses import dataclass

from analyst_klondike.state.base_action import BaseAction


@dataclass
class QuizNodeExpandAction(BaseAction):
    type = "QuizNodeExpand"
    quiz_id: str


@dataclass
class QuizNodeCollapseAction(BaseAction):
    type = "QuizNodeCollapsed"
    quiz_id: str



from typing import Literal
from textual import on
from textual.app import ComposeResult
from textual.screen import ModalScreen
from textual.containers import Vertical, Horizontal, Container
from textual.widgets import Label, Button

SaveOrExitModalResult = Literal["save", "not_save", "cancel"]


class SaveOnExitModal(ModalScreen[SaveOrExitModalResult]):

    CSS_PATH = "save_on_exit_modal.tcss"

    def compose(self) -> ComposeResult:
        yield Container(id="shadow")
        with Vertical(id="dialog"):
            yield Label("Сохранить файл?")
            with Horizontal(id="buttons"):
                yield Button(id="save", label="Сохранить", variant="success")
                yield Button(id="not_save", label="Не сохранять", variant="error")
                yield Button(id="cancel", label="Отмена", variant="default")

    @on(Button.Pressed, "#save")
    def on_save_button_clicked(self) -> None:
        self.dismiss("save")

    @on(Button.Pressed, "#not_save")
    def on_not_save_button_clicked(self) -> None:
        self.dismiss("not_save")

    @on(Button.Pressed, "#cancel")
    def on_cancel_button_clicked(self) -> None:
        self.dismiss("cancel")



from textwrap import dedent

from analyst_klondike.features.code_import.analysis.code_analysis import create_quiz_from_code


def test_code_analysis() -> None:
    code = dedent("""\
        def sum_values(a: int, b: int) -> int:
            '''
                Сумма двух значений
                Написать функцию, которая суммирует два числа.
                При этом нужно использовать только стандартные конструкции Python.
            '''
            return 1


        assert sum_values(1, 1) == 2
        assert sum_values(10, 15) == 25
        assert sum_values(-7, 3) == 4


        def compare_lists(lst1: list[int], lst2: list[int]) -> bool:
            '''
                Сравнение двух списков
                Написать функцию, которая сравнивает два массива.
            '''
            return True


        assert compare_lists([1, 2, 3], [1, 2, 3]) == True
        assert compare_lists([1, 2, 3], [1, 2, 3]) == True
        assert compare_lists([5, 6, 7], [5, 6, 7, 8]) == False
    """)
    quiz_state = create_quiz_from_code("python_quiz", code)
    print(quiz_state)


def test_negative_numbers_in_asserts():
    code = dedent("""\
        def sum_values(a: int, b: int) -> int:
            '''
                Сумма двух значений
                Написать функцию, которая суммирует два числа.
                При этом нужно использовать только стандартные конструкции Python.
            '''
            return 1
                          
        assert sum_values(-10, -15) == -25
        assert sum_values(1, 1) == 2
        assert sum_values(-7, 3) == 4
           
    """)
    quiz_state = create_quiz_from_code("python_quiz", code)
    print(quiz_state)


def test_false_in_asserts():
    code = dedent("""\
                 
        def compare_lists(lst1: list[int], lst2: list[int]) -> bool:
            '''
                Написать функцию, которая сравнивает два массива.
            '''
            return True

        assert compare_lists([1,2,3], [1,2,3,4]) == False      
        assert compare_lists([1,2,3], [1,2,3]) == True
           
    """)
    quiz_state = create_quiz_from_code("python_quiz", code)
    print(quiz_state)



import os
from analyst_klondike.features.code.actions import RunAllCodeAction
from analyst_klondike.features.data_context.init_action import InitAction
from analyst_klondike.features.data_context.json_load.dc import get_quiz_json
from analyst_klondike.features.data_context.set_opened_file_action import SetOpenedFileAction
from analyst_klondike.features.events.event_hook import send_event
from analyst_klondike.features.message_box.actions import HideMessageBoxAction
from analyst_klondike.state.app_dispatch import app_dispatch


def _demo_file_path() -> str:
    this_file_dir = os.path.dirname(os.path.realpath(__file__))
    return os.path.join(this_file_dir, "demo.yaml")


def start_demo_quiz():
    fpath = _demo_file_path()
    app_dispatch(SetOpenedFileAction(
        opened_file_name=os.path.basename(fpath),
        opened_file_path=fpath
    ))

    load_result = get_quiz_json(fpath)

    app_dispatch(InitAction(data=load_result))
    app_dispatch(RunAllCodeAction())
    app_dispatch(HideMessageBoxAction())
    send_event("start_demo_quiz", "/")
